@@ -17,6 +17,7 @@
 #include <libcamera/control_ids.h>
 #include <libcamera/controls.h>
 #include <libcamera/geometry.h>
+#include <libcamera/transform.h>
 
 #include <libcamera/ipa/core_ipa_interface.h>
 
@@ -29,8 +30,6 @@ class BayerFormat;
 class CameraLens;
 class MediaEntity;
 
-enum class Transform;
-
 struct CameraSensorProperties;
 
 class CameraSensor : protected Loggable
@@ -55,7 +54,9 @@ public:
 
 	V4L2SubdeviceFormat getFormat(const std::vector<unsigned int> &mbusCodes,
 				      const Size &size) const;
-	int setFormat(V4L2SubdeviceFormat *format);
+	int setFormat(V4L2SubdeviceFormat *format,
+		      Transform transform = Transform::Identity);
+	int tryFormat(V4L2SubdeviceFormat *format) const;
 
 	const ControlInfoMap &controls() const;
 	ControlList getControls(const std::vector<uint32_t> &ids);
