@@ -16,7 +16,6 @@
 #include <string.h>
 
 #include <libcamera/property_ids.h>
-#include <libcamera/transform.h>
 
 #include <libcamera/base/utils.h>
 
@@ -261,7 +260,7 @@ int CameraSensor::validateSensorDriver()
 		supportFlips_ = true;
 
 	if (!supportFlips_)
-		LOG(CameraSensor, Warning)
+		LOG(CameraSensor, Debug)
 			<< "Camera sensor does not support horizontal/vertical flip";
 
 	/*
@@ -462,7 +461,17 @@ int CameraSensor::initProperties()
 
 	const auto &rotationControl = controls.find(V4L2_CID_CAMERA_SENSOR_ROTATION);
 	if (rotationControl != controls.end()) {
+		/*
+		 * validateTransform() compensates for the mounting rotation.
+		 * However, as a camera sensor can only compensate rotations
+		 * by applying H/VFlips, only rotation of 180 degrees are
+		 * automatically compensated. The other valid rotations (Rot90
+		 * and Rot270) require transposition, which the camera sensor
+		 * cannot perform, so leave them untouched.
+		 */
 		propertyValue = rotationControl->second.def().get<int32_t>();
+		if (propertyValue == 180 && supportFlips_)
+			propertyValue = 0;
 		properties_.set(properties::Rotation, propertyValue);
 	}
 
@@ -751,7 +760,6 @@ V4L2SubdeviceFormat CameraSensor::getFormat(const std::vector<unsigned int> &mbu
 		.mbus_code = bestCode,
 		.size = *bestSize,
 		.colorSpace = ColorSpace::Raw,
-		.transform = Transform::Identity,
 	};
 
 	return format;
@@ -760,6 +768,8 @@ V4L2SubdeviceFormat CameraSensor::getFormat(const std::vector<unsigned int> &mbu
 /**
  * \brief Set the sensor output format
  * \param[in] format The desired sensor output format
+ * \param[in] transform The transform to be applied on the sensor.
+ * Defaults to Identity.
  *
  * If flips are writable they are configured according to the desired Transform.
  * Transform::Identity always corresponds to H/V flip being disabled if the
@@ -770,18 +780,16 @@ V4L2SubdeviceFormat CameraSensor::getFormat(const std::vector<unsigned int> &mbu
  *
  * \return 0 on success or a negative error code otherwise
  */
-int CameraSensor::setFormat(V4L2SubdeviceFormat *format)
+int CameraSensor::setFormat(V4L2SubdeviceFormat *format, Transform transform)
 {
 	/* Configure flips if the sensor supports that. */
 	if (supportFlips_) {
 		ControlList flipCtrls(subdev_->controls());
 
 		flipCtrls.set(V4L2_CID_HFLIP,
-			      static_cast<int32_t>(!!(format->transform &
-						      Transform::HFlip)));
+			      static_cast<int32_t>(!!(transform & Transform::HFlip)));
 		flipCtrls.set(V4L2_CID_VFLIP,
-			      static_cast<int32_t>(!!(format->transform &
-						      Transform::VFlip)));
+			      static_cast<int32_t>(!!(transform & Transform::VFlip)));
 
 		int ret = subdev_->setControls(&flipCtrls);
 		if (ret)
@@ -797,6 +805,23 @@ int CameraSensor::setFormat(V4L2SubdeviceFormat *format)
 	return 0;
 }
 
+/**
+ * \brief Try the sensor output format
+ * \param[in] format The desired sensor output format
+ *
+ * The ranges of any controls associated with the sensor are not updated.
+ *
+ * \todo Add support for Transform by changing the format's Bayer ordering
+ * before calling subdev_->setFormat().
+ *
+ * \return 0 on success or a negative error code otherwise
+ */
+int CameraSensor::tryFormat(V4L2SubdeviceFormat *format) const
+{
+	return subdev_->setFormat(pad_, format,
+				  V4L2Subdevice::Whence::TryFormat);
+}
+
 /**
  * \brief Retrieve the supported V4L2 controls and their information
  *
@@ -941,6 +966,9 @@ int CameraSensor::sensorInfo(IPACameraSensorInfo *info) const
 	info->bitsPerPixel = format.bitsPerPixel();
 	info->outputSize = format.size;
 
+	std::optional<int32_t> cfa = properties_.get(properties::draft::ColorFilterArrangement);
+	info->cfaPattern = cfa ? *cfa : properties::draft::RGB;
+
 	/*
 	 * Retrieve the pixel rate, line length and minimum/maximum frame
 	 * duration through V4L2 controls. Support for the V4L2_CID_PIXEL_RATE,
@@ -1010,10 +1038,15 @@ void CameraSensor::updateControlInfo()
  */
 Transform CameraSensor::validateTransform(Transform *transform) const
 {
-	/* Adjust the requested transform to compensate the sensor rotation. */
-	int32_t rotation = properties().get(properties::Rotation).value_or(0);
-	bool success;
+	/* Adjust the requested transform to compensate the sensor mounting rotation. */
+	const ControlInfoMap &controls = subdev_->controls();
+	int rotation = 0;
 
+	const auto &rotationControl = controls.find(V4L2_CID_CAMERA_SENSOR_ROTATION);
+	if (rotationControl != controls.end())
+		rotation = rotationControl->second.def().get<int32_t>();
+
+	bool success;
 	Transform rotationTransform = transformFromRotation(rotation, &success);
 	if (!success)
 		LOG(CameraSensor, Warning) << "Invalid rotation of " << rotation
@@ -1023,8 +1056,8 @@ Transform CameraSensor::validateTransform(Transform *transform) const
 
 	/*
 	 * We combine the platform and user transform, but must "adjust away"
-	 * any combined result that includes a transform, as we can't do those.
-	 * In this case, flipping only the transpose bit is helpful to
+	 * any combined result that includes a transposition, as we can't do
+	 * those. In this case, flipping only the transpose bit is helpful to
 	 * applications - they either get the transform they requested, or have
 	 * to do a simple transpose themselves (they don't have to worry about
 	 * the other possible cases).
