@@ -158,9 +158,7 @@ class CameraContext:
 
             print('Camera configuration adjusted')
 
-        r = self.camera.configure(camconfig)
-        if r != 0:
-            raise Exception('Configure failed')
+        self.camera.configure(camconfig)
 
         self.stream_names = {}
         self.streams = []
@@ -175,12 +173,7 @@ class CameraContext:
         allocator = libcam.FrameBufferAllocator(self.camera)
 
         for stream in self.streams:
-            ret = allocator.allocate(stream)
-            if ret < 0:
-                print('Cannot allocate buffers')
-                exit(-1)
-
-            allocated = len(allocator.buffers(stream))
+            allocated = allocator.allocate(stream)
 
             print('{}-{}: Allocated {} buffers'.format(self.id, self.stream_names[stream], allocated))
 
@@ -205,10 +198,7 @@ class CameraContext:
                 buffers = self.allocator.buffers(stream)
                 buffer = buffers[buf_num]
 
-                ret = request.add_buffer(stream, buffer)
-                if ret < 0:
-                    print('Can not set buffer for request')
-                    exit(-1)
+                request.add_buffer(stream, buffer)
 
             requests.append(request)
 
