@@ -191,6 +191,9 @@ class CommitFile:
         else:
             self.__filename = info[1]
 
+    def __repr__(self):
+        return f'{self.__status} {self.__filename}'
+
     @property
     def filename(self):
         return self.__filename
@@ -207,12 +210,22 @@ class Commit:
 
     def _parse(self):
         # Get the commit title and list of files.
-        ret = subprocess.run(['git', 'show', '--pretty=oneline', '--name-status',
+        ret = subprocess.run(['git', 'show', '--format=%s%n%(trailers:only,unfold)', '--name-status',
                               self.commit],
                              stdout=subprocess.PIPE).stdout.decode('utf-8')
-        files = ret.splitlines()
-        self._files = [CommitFile(f) for f in files[1:]]
-        self._title = files[0]
+        lines = ret.splitlines()
+
+        self._title = lines[0]
+
+        self._trailers = []
+        for index in range(1, len(lines)):
+            line = lines[index]
+            if not line:
+                break
+
+            self._trailers.append(line)
+
+        self._files = [CommitFile(f) for f in lines[index:] if f]
 
     def files(self, filter='AMR'):
         return [f.filename for f in self._files if f.status in filter]
@@ -221,6 +234,10 @@ class Commit:
     def title(self):
         return self._title
 
+    @property
+    def trailers(self):
+        return self._trailers
+
     def get_diff(self, top_level, filename):
         diff = subprocess.run(['git', 'diff', '%s~..%s' % (self.commit, self.commit),
                                '--', '%s/%s' % (top_level, filename)],
@@ -298,8 +315,10 @@ class CommitChecker(metaclass=ClassRegistry):
     # Class methods
     #
     @classmethod
-    def checkers(cls):
+    def checkers(cls, names):
         for checker in cls.subclasses:
+            if names and checker.__name__ not in names:
+                continue
             yield checker
 
 
@@ -353,13 +372,18 @@ class HeaderAddChecker(CommitChecker):
 
 
 class TitleChecker(CommitChecker):
-    prefix_regex = re.compile(r'[0-9a-f]+ (([a-zA-Z0-9_.-]+: )+)')
+    prefix_regex = re.compile(r'^([a-zA-Z0-9_.-]+: )+')
     release_regex = re.compile(r'libcamera v[0-9]+\.[0-9]+\.[0-9]+')
 
     @classmethod
     def check(cls, commit, top_level):
         title = commit.title
 
+        # Skip the check when validating staged changes (as done through a
+        # pre-commit hook) as there is no title to check in that case.
+        if isinstance(commit, StagedChanges):
+            return []
+
         # Ignore release commits, they don't need a prefix.
         if TitleChecker.release_regex.fullmatch(title):
             return []
@@ -378,7 +402,7 @@ class TitleChecker(CommitChecker):
             if not prefix:
                 continue
 
-            prefix = prefix.group(1)
+            prefix = prefix.group(0)
             if prefix in prefixes:
                 prefixes[prefix] += 1
             else:
@@ -414,6 +438,68 @@ class TitleChecker(CommitChecker):
                             'possible candidates are ' + candidates)]
 
 
+class TrailersChecker(CommitChecker):
+    commit_regex = re.compile(r'[0-9a-f]{12}[0-9a-f]* \(".*"\)')
+
+    coverity_regex = re.compile(r'Coverity CID=.*')
+
+    # Simple e-mail address validator regex, with an additional trailing
+    # comment. The complexity of a full RFC6531 validator isn't worth the
+    # additional invalid addresses it would reject.
+    email_regex = re.compile(r'[^<]+ <[^@>]+@[^>]+>( # .*)?')
+
+    link_regex = re.compile(r'https?://.*')
+
+    @staticmethod
+    def validate_reported_by(value):
+        if TrailersChecker.email_regex.fullmatch(value):
+            return True
+        if TrailersChecker.coverity_regex.fullmatch(value):
+            return True
+        return False
+
+    known_trailers = {
+        'Acked-by': email_regex,
+        'Bug': link_regex,
+        'Fixes': commit_regex,
+        'Link': link_regex,
+        'Reported-by': validate_reported_by,
+        'Reviewed-by': email_regex,
+        'Signed-off-by': email_regex,
+        'Suggested-by': email_regex,
+        'Tested-by': email_regex,
+    }
+
+    trailer_regex = re.compile(r'([A-Z][a-zA-Z-]*)\s*:\s*(.*)')
+
+    @classmethod
+    def check(cls, commit, top_level):
+        issues = []
+
+        for trailer in commit.trailers:
+            match = TrailersChecker.trailer_regex.fullmatch(trailer)
+            if not match:
+                raise RuntimeError(f"Malformed commit trailer '{trailer}'")
+
+            key, value = match.groups()
+
+            validator = TrailersChecker.known_trailers.get(key)
+            if not validator:
+                issues.append(CommitIssue(f"Invalid commit trailer key '{key}'"))
+                continue
+
+            if isinstance(validator, re.Pattern):
+                valid = bool(validator.fullmatch(value))
+            else:
+                valid = validator(value)
+
+            if not valid:
+                issues.append(CommitIssue(f"Malformed value '{value}' for commit trailer '{key}'"))
+                continue
+
+        return issues
+
+
 # ------------------------------------------------------------------------------
 # Style Checkers
 #
@@ -428,8 +514,10 @@ class StyleChecker(metaclass=ClassRegistry):
     # Class methods
     #
     @classmethod
-    def checkers(cls, filename):
+    def checkers(cls, filename, names):
         for checker in cls.subclasses:
+            if names and checker.__name__ not in names:
+                continue
             if checker.supports(filename):
                 yield checker
 
@@ -609,8 +697,10 @@ class Formatter(metaclass=ClassRegistry):
     # Class methods
     #
     @classmethod
-    def formatters(cls, filename):
+    def formatters(cls, filename, names):
         for formatter in cls.subclasses:
+            if names and formatter.__name__ not in names:
+                continue
             if formatter.supports(filename):
                 yield formatter
 
@@ -772,7 +862,7 @@ class StripTrailingSpaceFormatter(Formatter):
 # Style checking
 #
 
-def check_file(top_level, commit, filename):
+def check_file(top_level, commit, filename, checkers):
     # Extract the line numbers touched by the commit.
     commit_diff = commit.get_diff(top_level, filename)
 
@@ -789,7 +879,7 @@ def check_file(top_level, commit, filename):
     after = commit.get_file(filename)
 
     formatted = after
-    for formatter in Formatter.formatters(filename):
+    for formatter in Formatter.formatters(filename, checkers):
         formatted = formatter.format(filename, formatted)
 
     after = after.splitlines(True)
@@ -803,7 +893,7 @@ def check_file(top_level, commit, filename):
 
     # Check for code issues not related to formatting.
     issues = []
-    for checker in StyleChecker.checkers(filename):
+    for checker in StyleChecker.checkers(filename, checkers):
         checker = checker(after)
         for hunk in commit_diff:
             issues += checker.check(hunk.side('to').touched)
@@ -831,16 +921,17 @@ def check_file(top_level, commit, filename):
     return len(formatted_diff) + len(issues)
 
 
-def check_style(top_level, commit):
-    separator = '-' * len(commit.title)
+def check_style(top_level, commit, checkers):
+    title = commit.commit + ' ' + commit.title
+    separator = '-' * len(title)
     print(separator)
-    print(commit.title)
+    print(title)
     print(separator)
 
     issues = 0
 
     # Apply the commit checkers first.
-    for checker in CommitChecker.checkers():
+    for checker in CommitChecker.checkers(checkers):
         for issue in checker.check(commit, top_level):
             print('%s%s%s' % (Colours.fg(Colours.Yellow), issue.msg, Colours.reset()))
             issues += 1
@@ -852,7 +943,7 @@ def check_style(top_level, commit):
     files = [f for f in commit.files() if len([p for p in patterns if fnmatch.fnmatch(os.path.basename(f), p)])]
 
     for f in files:
-        issues += check_file(top_level, commit, f)
+        issues += check_file(top_level, commit, f, checkers)
 
     if issues == 0:
         print('No issue detected')
@@ -902,6 +993,8 @@ def main(argv):
 
     # Parse command line arguments
     parser = argparse.ArgumentParser()
+    parser.add_argument('--checkers', '-c', type=str,
+                        help='Specify which checkers to run as a comma-separated list. Defaults to all checkers')
     parser.add_argument('--staged', '-s', action='store_true',
                         help='Include the changes in the index. Defaults to False')
     parser.add_argument('--amend', '-a', action='store_true',
@@ -910,6 +1003,9 @@ def main(argv):
                         help='Revision range (as defined by git rev-parse). Defaults to HEAD if not specified.')
     args = parser.parse_args(argv[1:])
 
+    if args.checkers:
+        args.checkers = args.checkers.split(',')
+
     # Check for required dependencies.
     for command, mandatory in dependencies.items():
         found = shutil.which(command)
@@ -943,7 +1039,7 @@ def main(argv):
 
     issues = 0
     for commit in commits:
-        issues += check_style(top_level, commit)
+        issues += check_style(top_level, commit, args.checkers)
         print('')
 
     if issues:
